@@ -1,16 +1,19 @@
 //! # TLS parser
-//! Parsing functions for the TLS protocol, supporting versions 1.0 to 1.2
+//! Parsing functions for the TLS protocol, supporting versions 1.0 to 1.3
 
 use nom::{be_u8,be_u16,be_u24,be_u32,rest,IResult,Err,ErrorKind};
 
-use tls_alert::*;
-use tls_ciphers::*;
-use tls_ec::ECPoint;
+use crate::tls_alert::*;
+use crate::tls_ciphers::*;
+use crate::tls_ec::ECPoint;
 
 use std::ops::Deref;
 use std::convert::AsRef;
 use std::fmt;
 
+/// Max record size (RFC8446 5.1)
+pub const MAX_RECORD_LEN : u16 = 1 << 14;
+
 /// Handshake type
 ///
 /// Handshake types are defined in [RFC5246](https://tools.ietf.org/html/rfc5246) and
@@ -19,27 +22,28 @@ use std::fmt;
 #[derive(Clone, Copy, PartialEq, Eq)]
 pub struct TlsHandshakeType(pub u8);
 
-#[allow(non_upper_case_globals)]
-impl TlsHandshakeType {
-    pub const HelloRequest        : TlsHandshakeType = TlsHandshakeType(0x00);
-    pub const ClientHello         : TlsHandshakeType = TlsHandshakeType(0x01);
-    pub const ServerHello         : TlsHandshakeType = TlsHandshakeType(0x02);
-    pub const NewSessionTicket    : TlsHandshakeType = TlsHandshakeType(0x04);
-    pub const EndOfEarlyData      : TlsHandshakeType = TlsHandshakeType(0x05);
-    pub const HelloRetryRequest   : TlsHandshakeType = TlsHandshakeType(0x06);
-    pub const EncryptedExtensions : TlsHandshakeType = TlsHandshakeType(0x08);
-    pub const Certificate         : TlsHandshakeType = TlsHandshakeType(0x0b);
-    pub const ServerKeyExchange   : TlsHandshakeType = TlsHandshakeType(0x0c);
-    pub const CertificateRequest  : TlsHandshakeType = TlsHandshakeType(0x0d);
-    pub const ServerDone          : TlsHandshakeType = TlsHandshakeType(0x0e);
-    pub const CertificateVerify   : TlsHandshakeType = TlsHandshakeType(0x0f);
-    pub const ClientKeyExchange   : TlsHandshakeType = TlsHandshakeType(0x10);
-    pub const Finished            : TlsHandshakeType = TlsHandshakeType(0x14);
-    pub const CertificateURL      : TlsHandshakeType = TlsHandshakeType(0x15);
-    pub const CertificateStatus   : TlsHandshakeType = TlsHandshakeType(0x16);
-    pub const KeyUpdate           : TlsHandshakeType = TlsHandshakeType(0x18);
+newtype_enum!{
+impl debug TlsHandshakeType {
+    HelloRequest        = 0x00,
+    ClientHello         = 0x01,
+    ServerHello         = 0x02,
+    NewSessionTicket    = 0x04,
+    EndOfEarlyData      = 0x05,
+    HelloRetryRequest   = 0x06,
+    EncryptedExtensions = 0x08,
+    Certificate         = 0x0b,
+    ServerKeyExchange   = 0x0c,
+    CertificateRequest  = 0x0d,
+    ServerDone          = 0x0e,
+    CertificateVerify   = 0x0f,
+    ClientKeyExchange   = 0x10,
+    Finished            = 0x14,
+    CertificateURL      = 0x15,
+    CertificateStatus   = 0x16,
+    KeyUpdate           = 0x18,
 
-    pub const NextProtocol        : TlsHandshakeType = TlsHandshakeType(0x43);
+    NextProtocol        = 0x43,
+}
 }
 
 impl From<TlsHandshakeType> for u8 {
@@ -53,45 +57,27 @@ impl From<TlsHandshakeType> for u8 {
 #[derive(Clone, Copy, PartialEq, Eq)]
 pub struct TlsVersion(pub u16);
 
-#[allow(non_upper_case_globals)]
-impl TlsVersion {
-    pub const Ssl30        : TlsVersion = TlsVersion(0x0300);
-    pub const Tls10        : TlsVersion = TlsVersion(0x0301);
-    pub const Tls11        : TlsVersion = TlsVersion(0x0302);
-    pub const Tls12        : TlsVersion = TlsVersion(0x0303);
-    pub const Tls13        : TlsVersion = TlsVersion(0x0304);
+newtype_enum!{
+impl debug TlsVersion {
+    Ssl30        = 0x0300,
+    Tls10        = 0x0301,
+    Tls11        = 0x0302,
+    Tls12        = 0x0303,
+    Tls13        = 0x0304,
 
-    pub const Tls13Draft18 : TlsVersion = TlsVersion(0x7f12);
-    pub const Tls13Draft19 : TlsVersion = TlsVersion(0x7f13);
-    pub const Tls13Draft20 : TlsVersion = TlsVersion(0x7f14);
-    pub const Tls13Draft21 : TlsVersion = TlsVersion(0x7f15);
-    pub const Tls13Draft22 : TlsVersion = TlsVersion(0x7f16);
-    pub const Tls13Draft23 : TlsVersion = TlsVersion(0x7f17);
+    Tls13Draft18 = 0x7f12,
+    Tls13Draft19 = 0x7f13,
+    Tls13Draft20 = 0x7f14,
+    Tls13Draft21 = 0x7f15,
+    Tls13Draft22 = 0x7f16,
+    Tls13Draft23 = 0x7f17,
+}
 }
 
 impl From<TlsVersion> for u16 {
     fn from(v: TlsVersion) -> u16 { v.0 }
 }
 
-impl fmt::Debug for TlsVersion {
-    fn fmt(&self, fmt: &mut fmt::Formatter) -> fmt::Result {
-        match *self {
-            TlsVersion::Ssl30        => fmt.write_str("TlsVersion::Ssl30"),
-            TlsVersion::Tls10        => fmt.write_str("TlsVersion::Tls10"),
-            TlsVersion::Tls11        => fmt.write_str("TlsVersion::Tls11"),
-            TlsVersion::Tls12        => fmt.write_str("TlsVersion::Tls12"),
-            TlsVersion::Tls13        => fmt.write_str("TlsVersion::Tls13"),
-            TlsVersion::Tls13Draft18 => fmt.write_str("TlsVersion::Tls13Draft18"),
-            TlsVersion::Tls13Draft19 => fmt.write_str("TlsVersion::Tls13Draft19"),
-            TlsVersion::Tls13Draft20 => fmt.write_str("TlsVersion::Tls13Draft20"),
-            TlsVersion::Tls13Draft21 => fmt.write_str("TlsVersion::Tls13Draft21"),
-            TlsVersion::Tls13Draft22 => fmt.write_str("TlsVersion::Tls13Draft22"),
-            TlsVersion::Tls13Draft23 => fmt.write_str("TlsVersion::Tls13Draft23"),
-            _                        => write!(fmt, "{:x}", self.0),
-        }
-    }
-}
-
 impl fmt::LowerHex for TlsVersion {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         write!(f, "{:x}", self.0)
@@ -103,64 +89,42 @@ impl fmt::LowerHex for TlsVersion {
 #[derive(Clone, Copy, PartialEq, Eq)]
 pub struct TlsHeartbeatMessageType(pub u8);
 
-#[allow(non_upper_case_globals)]
-impl TlsHeartbeatMessageType {
-    pub const HeartBeatRequest  : TlsHeartbeatMessageType = TlsHeartbeatMessageType(0x1);
-    pub const HeartBeatResponse : TlsHeartbeatMessageType = TlsHeartbeatMessageType(0x2);
+newtype_enum!{
+impl debug TlsHeartbeatMessageType {
+    HeartBeatRequest  = 0x1,
+    HeartBeatResponse = 0x2,
+}
 }
 
 impl From<TlsHeartbeatMessageType> for u8 {
     fn from(v: TlsHeartbeatMessageType) -> u8 { v.0 }
 }
 
-impl fmt::Display for TlsHeartbeatMessageType {
-    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        write!(f, "{}", self.0)
-    }
-}
-
-impl fmt::Debug for TlsHeartbeatMessageType {
-    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        write!(f, "{}", self.0)
-    }
-}
-
 /// Content type, as defined in IANA TLS ContentType registry
 #[derive(Clone, Copy, PartialEq, Eq)]
 pub struct TlsRecordType(pub u8);
 
-#[allow(non_upper_case_globals)]
-impl TlsRecordType {
-    pub const ChangeCipherSpec : TlsRecordType = TlsRecordType(0x14);
-    pub const Alert            : TlsRecordType = TlsRecordType(0x15);
-    pub const Handshake        : TlsRecordType = TlsRecordType(0x16);
-    pub const ApplicationData  : TlsRecordType = TlsRecordType(0x17);
-    pub const Heartbeat        : TlsRecordType = TlsRecordType(0x18);
+newtype_enum!{
+impl debug TlsRecordType {
+    ChangeCipherSpec = 0x14,
+    Alert            = 0x15,
+    Handshake        = 0x16,
+    ApplicationData  = 0x17,
+    Heartbeat        = 0x18,
+}
 }
 
 impl From<TlsRecordType> for u8 {
     fn from(v: TlsRecordType) -> u8 { v.0 }
 }
 
-impl fmt::Debug for TlsRecordType {
-    fn fmt(&self, fmt: &mut fmt::Formatter) -> fmt::Result {
-        match *self {
-            TlsRecordType::ChangeCipherSpec => fmt.write_str("TlsRecordType::ChangeCipherSpec"),
-            TlsRecordType::Alert            => fmt.write_str("TlsRecordType::Alert"),
-            TlsRecordType::Handshake        => fmt.write_str("Handshake::Alert"),
-            TlsRecordType::ApplicationData  => fmt.write_str("Handshake::ApplicationData"),
-            TlsRecordType::Heartbeat        => fmt.write_str("Handshake::Heartbeat"),
-            _                               => write!(fmt, "{:x}", self.0),
-        }
-    }
-}
-
 #[derive(Clone, Copy, PartialEq, Eq)]
 pub struct TlsCompressionID(pub u8);
 
-#[allow(non_upper_case_globals)]
-impl TlsCompressionID {
-    pub const Null : TlsCompressionID = TlsCompressionID(0x00);
+newtype_enum!{
+impl debug TlsCompressionID {
+    Null = 0x00,
+}
 }
 
 impl From<TlsCompressionID> for u8 {
@@ -176,15 +140,6 @@ impl AsRef<u8> for TlsCompressionID {
     fn as_ref(&self) -> &u8 { &self.0 }
 }
 
-impl fmt::Debug for TlsCompressionID {
-    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        match *self {
-            TlsCompressionID::Null => f.write_str("TlsCompressionID::Null"),
-            _                      => write!(f, "{}", self.0)
-        }
-    }
-}
-
 #[derive(Clone, Copy, PartialEq, Eq)]
 pub struct TlsCipherSuiteID(pub u16);
 
@@ -394,10 +349,12 @@ pub struct TlsNextProtocolContent<'a> {
     pub padding: &'a[u8],
 }
 
-enum_from_primitive! {
 /// Key update request (TLS 1.3)
-#[repr(u8)]
-pub enum KeyUpdateRequest {
+#[derive(Copy, Clone, PartialEq, Eq)]
+pub struct KeyUpdateRequest(pub u8);
+
+newtype_enum!{
+impl KeyUpdateRequest {
     NotRequested  = 0x0,
     Requested     = 0x1,
 }
@@ -449,7 +406,7 @@ pub struct TlsMessageHeartbeat<'a>{
 #[derive(Clone,PartialEq)]
 pub struct TlsRecordHeader {
     pub record_type: TlsRecordType,
-    pub version: u16,
+    pub version: TlsVersion,
     pub len: u16,
 }
 
@@ -519,6 +476,15 @@ fn parse_compressions_algs(i:&[u8], len:usize) -> IResult<&[u8],Vec<TlsCompressi
     Ok((&i[len..],v))
 }
 
+pub(crate) fn parse_tls_versions(i:&[u8]) -> IResult<&[u8],Vec<TlsVersion>> {
+    let len = i.len();
+    if len == 0 { return Ok((i,Vec::new())) }
+    if len%2 == 1 || len > i.len() { return Err(Err::Error(error_position!(i, ErrorKind::LengthValue))); }
+    let v = (&i[..len]).chunks(2).map(|chunk| {
+                            TlsVersion((chunk[0] as u16) << 8 | chunk[1] as u16) }).collect();
+    Ok((&i[len..],v))
+}
+
 named!(parse_certs<Vec<RawCertificate> >,
     many0!(
         complete!(
@@ -538,7 +504,7 @@ named!(parse_tls_record_header<TlsRecordHeader>,
         (
             TlsRecordHeader {
                 record_type: TlsRecordType(t),
-                version: v,
+                version: TlsVersion(v),
                 len: l,
             }
         )
@@ -752,9 +718,9 @@ fn parse_tls_handshake_msg_finished( i:&[u8], len: usize ) -> IResult<&[u8], Tls
     )
 }
 
-/// Defined in [RFC6066]
-/// if status_type == 0, blob is a OCSPResponse, as defined in [RFC2560](https://tools.ietf.org/html/rfc2560)
-/// Note that the OCSPResponse object is DER-encoded.
+// Defined in [RFC6066]
+// if status_type == 0, blob is a OCSPResponse, as defined in [RFC2560](https://tools.ietf.org/html/rfc2560)
+// Note that the OCSPResponse object is DER-encoded.
 named!(parse_tls_handshake_msg_certificatestatus<TlsMessageHandshake>,
     do_parse!(
         status_type: be_u8 >>
@@ -836,8 +802,8 @@ named!(parse_tls_message_alert<TlsMessage>,
         c: be_u8 >>
         ( TlsMessage::Alert(
                 TlsMessageAlert {
-                    severity: s,
-                    code: c,
+                    severity: TlsAlertSeverity(s),
+                    code: TlsAlertDescription(c),
             }
         ) )
     )
@@ -854,19 +820,20 @@ fn parse_tls_message_applicationdata( i:&[u8] ) -> IResult<&[u8], TlsMessage> {
     })
 }
 
-fn parse_tls_message_heartbeat( i:&[u8] ) -> IResult<&[u8], TlsMessage> {
+fn parse_tls_message_heartbeat(i:&[u8], tls_plaintext_len:u16) -> IResult<&[u8], Vec<TlsMessage>> {
     do_parse!(i,
         hb_type: be_u8 >>
         hb_len: be_u16 >>
-        b: take!(i.len()-3) >> // payload (hb_len) + padding
+           error_if!(tls_plaintext_len < 3, ErrorKind::Verify) >>
+        b: take!(tls_plaintext_len - 3) >> // payload (hb_len) + padding
         (
-            TlsMessage::Heartbeat(
+            vec![TlsMessage::Heartbeat(
                 TlsMessageHeartbeat {
                     heartbeat_type: TlsHeartbeatMessageType(hb_type),
                     payload_len: hb_len,
                     payload: b,
                 }
-            )
+            )]
         )
     )
 }
@@ -883,7 +850,7 @@ pub fn parse_tls_record_with_header( i:&[u8], hdr:TlsRecordHeader ) -> IResult<&
         TlsRecordType::Alert            => many1!(i, complete!(parse_tls_message_alert)),
         TlsRecordType::Handshake        => many1!(i, complete!(parse_tls_message_handshake)),
         TlsRecordType::ApplicationData  => many1!(i, complete!(parse_tls_message_applicationdata)),
-        TlsRecordType::Heartbeat        => many1!(i, complete!(parse_tls_message_heartbeat)),
+        TlsRecordType::Heartbeat        => parse_tls_message_heartbeat(i, hdr.len),
         _                               => Err(Err::Error(error_position!(i, ErrorKind::Switch)))
     }
 }
@@ -894,6 +861,7 @@ pub fn parse_tls_record_with_header( i:&[u8], hdr:TlsRecordHeader ) -> IResult<&
 pub fn parse_tls_plaintext(i:&[u8]) -> IResult<&[u8],TlsPlaintext> {
     do_parse!(i,
         hdr: parse_tls_record_header >>
+             error_if!(hdr.len > MAX_RECORD_LEN, ErrorKind::TooLarge) >>
         msg: flat_map!(take!(hdr.len),
             apply!(parse_tls_record_with_header,hdr.clone())
             ) >>
@@ -905,6 +873,7 @@ pub fn parse_tls_plaintext(i:&[u8]) -> IResult<&[u8],TlsPlaintext> {
 pub fn parse_tls_encrypted(i:&[u8]) -> IResult<&[u8],TlsEncrypted> {
     do_parse!(i,
         hdr: parse_tls_record_header >>
+             error_if!(hdr.len > MAX_RECORD_LEN, ErrorKind::TooLarge) >>
         blob: take!(hdr.len) >>
         ( TlsEncrypted {hdr:hdr, msg:TlsEncryptedContent{ blob: blob}} )
     )
@@ -918,6 +887,7 @@ pub fn parse_tls_encrypted(i:&[u8]) -> IResult<&[u8],TlsEncrypted> {
 pub fn parse_tls_raw_record(i:&[u8]) -> IResult<&[u8],TlsRawRecord> {
     do_parse!(i,
         hdr: parse_tls_record_header >>
+             error_if!(hdr.len > MAX_RECORD_LEN, ErrorKind::TooLarge) >>
         data: take!(hdr.len) >>
         ( TlsRawRecord {hdr:hdr, data: data} )
     )
